from variables import BASE, REF, FARM, CAM
from gwlogging import logger
from GwErrors import GwNotImplementedError, GwNotApplicableError, \
    GwModeError


class conversions:
    __mode__ = None

    def __init__(self, mode):
        self.__mode__ = mode
        logger.debug("Conversions setup for mode %s" % mode)

    def batV(self, adc):
        # (30.06 + 160)/30.06 * 2.5 / 4095
        if self.__mode__ is BASE:
            reading = float(adc) * 0.0036 + 0.1085
        elif self.__mode__ is REF:
            reading = float(adc) * 0.004 + 0.1259
        elif self.__mode__ is CAM:
            reading = float(adc) * 0.0043 + 0.0738
        elif self.__mode__ is FARM:
            raise GwNotApplicableError()
        else:
            raise GwModeError()
        return reading

    def solar_v(self, adc):
        if self.__mode__ is BASE:
            reading = float(adc) * 0.0082
        elif self.__mode__ is REF:
            reading = float(adc) * 0.0082
        elif self.__mode__ is CAM:
            raise GwNotImplementedError()
        elif self.__mode__ is FARM:
            raise GwNotApplicableError()
        else:
            raise GwModeError()
        return reading

    def wind_v(self, adc):
        if self.__mode__ is BASE:
            reading = float(adc) * 0.0083
        elif self.__mode__ is REF:
            reading = float(adc) * 0.0064 + 0.9212
        elif self.__mode__ is CAM:
            raise GwNotImplementedError()
        elif self.__mode__ is FARM:
            raise GwNotApplicableError()
        else:
            raise GwModeError()
        return reading

    def cafe_current(self, adc):
        if self.__mode__ is BASE:
            reading = float(adc) * 0.0071 - 0.0048
        elif self.__mode__ is REF:
            raise GwNotImplementedError()
        elif self.__mode__ is CAM:
            raise GwNotApplicableError()
        elif self.__mode__ is FARM:
            raise GwNotApplicableError()
        else:
            raise GwModeError()
        return reading

    def solar_current(self, adc):
        if self.__mode__ is BASE:
            reading = float(adc) * 0.0035 - 0.0078
        elif self.__mode__ is REF:
            raise GwNotImplementedError()
        elif self.__mode__ is CAM:
            raise GwNotApplicableError()
        elif self.__mode__ is FARM:
            raise GwNotApplicableError()
        else:
            raise GwModeError()
        return reading

    def wind_current(self, adc):
        if self.__mode__ is BASE:
            reading = float(adc) * 0.0061 + 0.0178
        elif self.__mode__ is REF:
            raise GwNotImplementedError()
        elif self.__mode__ is CAM:
            raise GwNotApplicableError()
        elif self.__mode__ is FARM:
            raise GwNotApplicableError()
        else:
            raise GwModeError()
        return reading

    def temperature(self, adc):
        if self.__mode__ is BASE:
            raise GwNotImplementedError()
        elif self.__mode__ is REF:
            cal = float(adc) * 0.0968 - 74.09
        elif self.__mode__ is CAM:
            cal = float(adc) * 0.077 - 47.213
        elif self.__mode__ is FARM:
            raise GwNotApplicableError()
        else:
            raise GwModeError()
        return round(cal, 1)
