# Handle GPRS connections

from datetime import datetime
from subprocess import Popen 
from time import sleep

from comms import disconnect_ppp, ppp_is_up
from variables import COMMS_TRY_TIME, GPRS_BAUD, GWGSM, GPRS_PORT, \
    PPP_PID_FILE
from gwlogging import logger, gw_stdout, gw_stderr
    

def init(use_gsmat=False):
    power_time = datetime.utcnow()
    if use_gsmat:
        logger.error("gsmat not available on this build")
        return False
    else:
        logger.info("Waiting for GPRS registration")
        while not registered():
            now = datetime.utcnow()
            if power_time + COMMS_TRY_TIME < now:
                logger.error("Failed to register GPRS - giving up")
                return False
            else:
                logger.info("Still waiting for GPRS registration")
                sleep(5)
        return True


def connect():
    p = Popen("/usr/sbin/pppd call glacs-isp %i" % GPRS_BAUD,
                          shell=True, stdout=gw_stdout, stderr=gw_stderr)
    r = p.wait()
    if r != 0:
        logger.error("Error!  pppd failed with exit status %i" % r)
        return False
    start_time = datetime.utcnow()
    logger.info("Waiting for GPRS PPP interface to come up")
    sleep(5)
    while not up():
        now = datetime.utcnow()
        if start_time + COMMS_TRY_TIME < now:
            logger.error(("Failed to get a GPRS connection within timelimit" +
                    " - giving up"))
            return False
        else:
            logger.info("Still waiting for GPRS PPP interface to come up")
            sleep(5)
    return True


def registered():
    """Returns True if the GPRS has registered with a GPRS service"""
    p = Popen("%s -b %i -p %s check-gprs" % (GWGSM, GPRS_BAUD,
                        GPRS_PORT),
                        shell=True, stdout=gw_stdout, stderr=gw_stderr)

    r = p.wait()

    return (r == 0)


def wait_for_registration():
    """Blocks until the modem has registered with a GPRS service"""
    logger.info("Waiting for GPRS registration...")
    while not registered():
        sleep(1)
        logger.info("Still waiting.")
    logger.info("GSM modem registered with GPRS service")


def disconnect():
    return disconnect_ppp(PPP_PID_FILE)


def up():
    return ppp_is_up(PPP_PID_FILE)
