#!/usr/bin/env python
# 2.2 changing to use icecape and logging instead of icInter and printf
# v2.1
# 2.1 more tidying and rearranging
# 2.0 start of code for 2011
# This script is runs on both the reference and base stations.

# Usage: python gwstation.py [-dt] -[B|R]
# -d    - Daemon mode
# -t     - testing mode
# -B    - Base station mode (enables probes)
# -R    - Reference station mode (disables probes)
# -F    - Farm station mode (very simple only does comms
#         stuff does not do any power management)
# -c    - Adds camera functionality to whatever mode it's running in

# Import required python libs
import datetime # TODO formalise
from sys import argv 
import time # TODO formalise
from time import sleep
#import our libraries that are definately needed
from variables import BASE, CAM, FARM, REF, BASE_DIR, DATA_DIR, \
    BASE_REVERSE_PORT, REF_REVERSE_PORT, FARM_REVERSE_PORT, CAM_REVERSE_PORT, \
    RUN_HOUR, RUN_MINUTE, RUN_SECOND, WORTH_POWER_OFF, BORECAM_CONF_FILE, \
    MSP_DATA_FILE, REF_NETWORK_INTERFACE, BASE_NETWORK_INTERFACE, \
    COMMS_TRY_TIME, CAM_NETWORK_INTERFACE 
from gwlogging import logger, daemonize
import gwutil #TODO formalise
from GwErrors import GwModeError

start_time = datetime.datetime.utcnow()
logger.info("run at %s" % start_time)

# Process argv
__ignore_times__ = False
__testing__ = False
__daemonize__ = False
__use_gsmat__ = False
__mode__ = None
__camera__ = False
__comms__ = True
for arg in argv[1:]:
    if arg == '-t':
        logger.warn("TESTING MODE ACTIVATED")
        __testing__ = True
    elif arg == '-d':
        __daemonize__ = True
    elif arg == '-m':
        logger.error("GSMAT IS DEPRECATED")
        __use_gsmat__ = True
    elif arg == "-i":
        logger.warn("IGNORING TIME CONSTRAINTS")
        __ignore_times__ = True
    elif arg == "-c":
        logger.info("Camera capture added")
        __camera__ = True
    elif arg == '-B':
        logger.info("Base mode activated")
        __mode__ = BASE
    elif arg == '-R':
        logger.info("Ref mode activated")
        __mode__ = REF
    elif arg == '-C':
        logger.info("Cam mode activated")
        __mode__ = CAM
        __camera__ = True
        __comms__ = True
    elif arg == '-F':
        logger.info("farm mode activated")
        __mode__ = FARM
    else:
        print "Useage: %s [-dt] -[B|R]" % (argv[0])
        print "\t-d: Daemonize process"
        print "\t-t: Testing mode - inhibits powering off"
        print "\t-i: Ignore timing constraints"
        print "\t-c: Adds camera capture into mode"
        print "\t-C: Camera only mode"
        print "\t-B: Base station mode - enables probes"
        print "\t-R: Reference station mode - disables probes"
        print ("\t-F: Farm station mode - only does comms stuff does not" +
                        " do any power management")
        exit(2)

if __mode__ is None:
    logger.critical("No mode set use flags -B, -F, -C  or -R")
    exit(1)

# Daemonize the process
if __daemonize__:
    # print to this file
    LOG_FILE = BASE_DIR + DATA_DIR + "/gw%s" % __mode__
    daemonize(LOG_FILE)
    logger.debug("Sucessfully daemonised")

gwutil.increment_boot_count()

# Set up reverse port variables and if __mode__ is BASE:
if __mode__ is BASE:
    SSH_REVERSE_PORT = BASE_REVERSE_PORT
elif __mode__ is REF:
    SSH_REVERSE_PORT = REF_REVERSE_PORT
elif __mode__ is FARM:
    SSH_REVERSE_PORT = FARM_REVERSE_PORT
elif __mode__ is CAM:
    SSH_REVERSE_PORT = CAM_REVERSE_PORT
else:
    logger.critical("Should not have been possible to get here")
    raise GwModeError()
logger.debug("SSH Port:%d" % SSH_REVERSE_PORT)


# Waiting due to wait file
while gwutil.wait():
    logger.info("Waiting due to wait file.")
    time.sleep(10)

#Branch to a different file for farm modeas it's much simpler
if __mode__ is FARM:
    import gwfarm
    gwfarm.run()
    logger.info("Finished running farm mode")
    exit(0)

# Import required custom libs
import gwschedule
import iceCape
import gwlevels
import MspData
import beagleIo


if __comms__:
    import comms
    if __mode__ is not CAM:
        import gprs

ic = iceCape.iceCape()
b = beagleIo.beagleIo()

# Check for power loss
last_run_time = gwutil.get_last_run_time()
now = datetime.datetime.utcnow()

if last_run_time is None:
    logger.error("Something went wrong reading the info file")
    logger.debug("Setting last run time to 2005")
    #2005 chosen so that it is later than the 0 epoch and the
    #1/1/00 that overo resets to
    # this means that if the rtc is also wrong a reset will be triggered
    last_run_time = datetime.datetime(2005, 1, 1, 0, 0, 0)
    gwlevels.reset(ic)
logger.info("Awake at %s, last run time was %s" % (now, last_run_time))

#ic should keep rtc so need to check if msp has correct rtc
# before resorting to gps

if now < last_run_time:
    logger.error("Current time is set in the past relative to the lastrun")
    gwschedule.set_local_state(ic, __mode__, gwlevels.CRITICAL)
    if(not gwutil.recover_time(ic, last_run_time)):
        logger.error("Failed to set time from MSP or GPS, will try tomorrow.")
        logger.info("Powering down")
        gwutil.handle_power_off(ic, start_time, __testing__)
        exit(1)
    else:
        logger.info("Date time sucessfully set")

if not gwutil.check_time(ic.get_datetime()):
    if not gwutil.check_time_coarse(ic.get_datetime()):
        logger.error("Assume msp lost everything")
        gwlevels.reset(ic)
        gwschedule.set_local_state(ic, __mode__, gwlevels.CRITICAL)
        logger.info("Schedule reset to 0")
        # has to be done first otherwise any
        #jobs run lots of time to catch up with new date causing issues...
        ic.set_epoch()
        logger.info("MSP date reset %s" % datetime.datetime.utcnow())
    else:
        ic.set_epoch()

# Check to see we are running at the correct time.
now = datetime.datetime.utcnow()
run_time = datetime.datetime(now.year, now.month, now.day,
                   RUN_HOUR, RUN_MINUTE, RUN_SECOND)
if not __ignore_times__:
    # Unless we wake up in 2 minutes, turn off.
    if now == run_time:
        logger.info("Awake at run time")
    elif  now < run_time:
        # Up before run time
        if (now + WORTH_POWER_OFF) < run_time:
            logger.error("Woken too early, going to sleep")
            logger.info("Powering down")
            gwutil.handle_power_off(ic, start_time, __testing__)
            exit(2)
        else:
            diff = run_time - now
            if diff.seconds > 0:
                logger.error("Up slightly early, waiting for %i seconds" % (
                       diff.seconds))
                time.sleep(diff.seconds)
                logger.info("Resuming in %s mode" % __mode__)
    else:
        # Up after run time
        # Check to see if we already ran today
        if (last_run_time.year == run_time.year
               and last_run_time.month == run_time.month
               and last_run_time.day == run_time.day):
            logger.error("Already executed today")
            logger.debug("Powering down")
            gwutil.handle_power_off(ic, start_time, __testing__)
            exit(3)
        else:
            logger.error("Running late!")
else:
    logger.warning("Ignoring times so running anyway")


# Get probe data this will only execute at 11:59:30 or later
if __mode__ is BASE:
    logger.info("Doing probey type things")
    gwutil.get_probe_data()
    gwutil.executeProbeSpecial()
    logger.info("Probey stuff complete")
    

if __camera__:
    logger.info("Grabbing image")
    if __mode__ is CAM:
        sleep(30)
        gwutil.get_camera_image(BORECAM_CONF_FILE)
    else:
        gwutil.get_camera_image()


#fetch all data from ic
mspdata = ic.read_data()
MspData.tofile(MSP_DATA_FILE, mspdata)


logger.info("Determining local power state")
state = gwschedule.set_local_state(ic, __mode__)


# If we have enough power to get data from other sensors
# and not running in camera mode
if state > gwlevels.LOW and __mode__ is not CAM:
    logger.info("Fetching GPS log files.")
    ic.turn_on_power_output(iceCape.GPS)
    gwutil.get_gps_logs(ic)
    ic.turn_off_power_output(iceCape.GPS)
    if state == 3:
        ic.turn_on_power_output(iceCape.GPS)
        #TODO HACK
        #should be done in a better way
        #enforced power cycle to restart logging
    if __mode__  is  BASE:
        logger.info("Fetching Geophone data")
        gwutil.get_geophone_data()
        gwutil.executeGeophoneSpecial()
    elif __mode__ is REF:
        logger.info("Fetching weather data")
        gwutil.get_weather_data(ic)

gwutil.packFiles(__mode__)

# Continue only if we have the power
# and are scheduled to do comms
if state > gwlevels.CRITICAL and __comms__:

    iface = None
    connected = None

    if state == gwlevels.HIGH or __mode__ is CAM:
        logger.info("Starting WiFi stuff")
        if(__mode__ is REF):
            logger.debug("No need to power anything as powered by schedule")
            iface = REF_NETWORK_INTERFACE
        elif __mode__ is CAM:
            logger.debug("Powering on AP")
            ic.turn_on_power_output(iceCape.CAM_AP)
            iface = CAM_NETWORK_INTERFACE
            sleep(60)
        else:
            logger.info("powered up WiFi stick")
            b.on(beagleIo.WIFI)
            iface = BASE_NETWORK_INTERFACE
        wifi_start_time = datetime.datetime.utcnow()
        while datetime.datetime.utcnow() < (wifi_start_time + COMMS_TRY_TIME):

            if comms.interface_up(iface):
                logger.info("Interface up")
                connected = "WIFI"
                break
            else:
                comms.interface_down(iface)
                time.sleep(2)

        if connected == None:
            if(__mode__ is REF):
                logger.debug("No poweroffs requried as powered by schedule")
            elif __mode__ is CAM:
                logger.info("turning off ap")
                ic.turn_off_power_output(iceCape.CAM_AP)
            else:
                logger.info("Powering off WiFi stick")
                b.off(beagleIo.WIFI)

    if connected is None and __mode__ is not CAM:
        logger.info("Powering on GPRS modem")

        # Power on the gprs modem
        ic.turn_on_power_output(iceCape.GPRS)

        # Give the modem a few seconds to power up
        time.sleep(20)
        if not gprs.init(__use_gsmat__):
            logger.error("Failed to connect to networks")
            #power off gprs modem
            ic.turn_off_power_output(iceCape.GPRS)
        else:
            logger.info("GSM modem registered with GPRS service")
            time.sleep(10)
            gprs.connect()

            if gprs.up():
                logger.info("GPRS interface up")
                connected = "GPRS"

    if connected is not None:
        #Use connection
        webServer_name = comms.chooseWebServer()
        sshServer_name = comms.chooseSshServer(webServer_name,
                           connected == "GPRS")

        if (webServer_name is None
                and sshServer_name is None
                and connected == "GPRS"):
            logger.error("No servers found when on WiFi falling back to GPRS")

            comms.interface_down(iface)
            time.sleep(2)
            if(__mode__ is REF):
                logger.debug("No poweroffs requried as powered by schedule")
            elif __mode__ is CAM:
                logger.info("turning off ap")
                ic.turn_off_power_output(iceCape.CAM_AP)
            else:
                logger.info("Powering off WiFi stick")
                b.off(beagleIo.WIFI)
            # Power on the gprs modem
            ic.turn_on_power_output(iceCape.GPRS)

            # Give the modem a few seconds to power up
            time.sleep(20)
            if not gprs.init(__use_gsmat__):
                logger.error("Failed to connect to networks")
                #power off gprs modem
                ic.turn_off_power_output(iceCape.GPRS)
            else:
                logger.info("GSM modem registered with GPRS service")
                time.sleep(10)
                gprs.connect()

            if gprs.up():
                logger.info("GPRS interface up")
                connected = "GPRS"

        if connected:
            webServer_name = comms.chooseWebServer()
            sshServer_name = comms.chooseSshServer(webServer_name,
                           connected == "GPRS")
            

        #Send the power state
        if webServer_name is not None and __mode__ is not CAM:
            comms.sendPowerState(webServer_name, __mode__, state)
        else:
            logger.error("No webserver avilable not sending power state")
        
        #Send  the files
        if sshServer_name is not None:
            logger.info("Starting upload")
            comms.uploadFiles(sshServer_name, False, connected)
            logger.info("Finished upload")
        else:
            logger.error("No ssh server avilable not sending data")

        #Get the over-ride power state and the special file if it exists

        if webServer_name is not None and __mode__ is not CAM:
            comms.fetchOverridePowerState(webServer_name, __mode__)
            # Set the new power state.
            logger.info("Setting new power state")
            gwschedule.set_state(ic, __mode__)
        else:
            logger.error(
                "Not got power state/special either no webserver/in cam mode")

        if webServer_name is not None:
            comms.getMainSpecial(webServer_name, __mode__)
            gwutil.executeSpecial()
            if __mode__ == BASE:
                logger.info("Mode is base so getting other special files")
                comms.getGeophoneSpecial(webServer_name)
                comms.getProbeSpecial(webServer_name)
            else:
                logger.info(
                    "Mode isn't base so not getting geophone/probe special")
        else:
            logger.error("No webserver not getting specials")


        #Open the tunnel back to the server
        if sshServer_name is not None:
            comms.open_ssh_tunnel(sshServer_name, SSH_REVERSE_PORT)
        else:
            logger.error("No ssh server avilable not openning tunnel")

        if connected is "GPRS":
            # Disconnect GPRS and turn off modem.
            logger.info("Disconnecting GPRS")
            gprs.disconnect()
            ic.turn_off_power_output(iceCape.GPRS)
        else:
            if(__mode__ is REF):
                logger.debug("everything powered by schedule")
            elif __mode__ is CAM:
                logger.info("turning off ap")
                ic.turn_off_power_output(iceCape.CAM_AP)
            else:
                logger.info("Powering off WiFi stick")
                ic.turn_off_power_output(iceCape.WIFI)
    else:
        logger.error("Both GPRS and WiFi connections failed")
        logger.info("Stopping Comms")
elif __comms__:
    logger.info("State %i not high enough for comms" % state)
else:
    logger.info("Not scheduled to do comms")


if __mode__ is BASE:
    try:
        from send_status import send_status
        send_status()
    except Exception:
        logger.error("Somethign went wrong with sat phone")
# If the script is not in testing mode
# then set the last time the script
# executed and sync the file system.
# Note: The next wake up time is called
#        at the beginning of the script
#        incase of power loss or SIGKILL
if not __testing__:
    logger.debug("Saving last run time")
    gwutil.set_last_run_time()
else:
    logger.warn("Run in debug mode - not setting last run time")

gwutil.handle_power_off(ic, start_time, __testing__)
