import MspJob
from datetime import time, timedelta
from  variables import BASE, CAM, REF 
import iceCapeMapping
from gwlogging import logger
from GwErrors import GwLevelError, GwModeError

CRITICAL = 0
LOW = 1
OK = 2
HIGH = 3

#Lower thresholds for levels

#Ignore anything less than this
IGNORE_V = 6.0
CRITICAL_V = 0

LOW_V = 11.5
OK_V = 12.5
HIGH_V = 13.5

thresholds = {HIGH: HIGH_V, OK: OK_V, LOW: LOW_V, CRITICAL: CRITICAL_V}

def fake_base(gum, mode):
    if mode is BASE:
        gum.add_job(MspJob.J_P_PCON,
            time(hour=11, minute=58),
            timedelta(days=1),
            timedelta(minutes=30),
            iceCapeMapping.PROBE_BASE,
            None)
        logger.info("Fake base noon job added")
        gum.add_job(MspJob.J_P_PCON,
            time(hour=18, minute=45),
            timedelta(days=1),
            timedelta(hours=1, minutes=30),
            iceCapeMapping.PROBE_BASE,
            None)
        logger.info("Fake base evening job added")
    else:
        logger.debug("Mode not base, not added fake base jobs")

def setup(gum, level, mode):
    #generic setup for all levels
    if(mode == BASE):
        gum.turn_on_sensor_output(iceCapeMapping.PROBE_1 + iceCapeMapping.PROBE_2)
        logger.debug("Probe power turned on")

    gum.clear_job_table()

    if mode == CAM:
        gum.add_job(MspJob.J_P_GumstixPcon,
            time(hour=12),
            timedelta(days=1),
            None,
            None,
            None)
        logger.warn("Only doing one wake up for camera node")
#        gum.add_job(MspJob.J_P_GumstixPcon,
#            time(hour=10),
#            timedelta(days=1),
#            None,
#            None,
#            None)
#        gum.add_job(MspJob.J_P_GumstixPcon,
#            time(hour=14),
#            timedelta(days=1),
#            None,
#            None,
#            None)
    else:
        gum.add_job(MspJob.J_P_GumstixPcon,
            time(hour=11, minute=59),
            timedelta(days=1),
            None,
            None,
            None)
    logger.info("Gumstix job added")

    if mode ==  CAM:
        gum.add_job(MspJob.J_P_GumstixPCON_OFF,
            time(hour=10, minute=10),
            timedelta(days=1),
            None,
            None,
            None)
        gum.add_job(MspJob.J_P_GumstixPCON_OFF,
            time(hour=12, minute=10),
            timedelta(days=1),
            None,
            None,
            None)
        gum.add_job(MspJob.J_P_GumstixPCON_OFF,
            time(hour=14, minute=10),
            timedelta(days=1),
            None,
            None,
            None)
    else:
        gum.add_job(MspJob.J_P_GumstixPCON_OFF,
            time(hour=14),
            timedelta(days=1),
            None,
            None,
            None)
    logger.info("Gumstix OFF job added")

    sensors_list = (iceCapeMapping.BATT_V + iceCapeMapping.TEMPERATURE +
        iceCapeMapping.X + iceCapeMapping.Y + iceCapeMapping.Z)
    if(mode == REF):
        sensors_list = (sensors_list + iceCapeMapping.CAFE + iceCapeMapping.REF_SOLAR +
            iceCapeMapping.SOLAR_V + iceCapeMapping.CAFE_V)
    elif(mode == BASE):
        sensors_list = (sensors_list + iceCapeMapping.WIND + iceCapeMapping.SOLAR +
            iceCapeMapping.SOLAR_V + iceCapeMapping.WIND_V)
    elif mode == CAM:
        pass
    else:
        logger.error("NOT SURE HOW I GOT HERE")
        raise GwModeError()

    gum.add_job(MspJob.J_S_SENSSPCON,
        time(hour=0, minute=30),
        timedelta(minutes=30),
        timedelta(seconds=2),
        iceCapeMapping.SENSE_EN + iceCapeMapping.BATT_EN,
        sensors_list)
    logger.info("Sensors job added")

    # Set up jobs table
    if level == CRITICAL:
        logger.info("Installing critical jobs table")
    elif level == LOW:
        logger.info("Installing low jobs table")
        fake_base(gum, mode)
    elif level == OK:
        logger.info("Installing ok jobs table")
        fake_base(gum, mode)
        if mode == BASE:# or mode == REF:
            logger.warn("GPS job removed from REF")
            gum.add_job(MspJob.J_P_PCON,
                time(hour=14),  # 1970-01-01 14:00:00
                timedelta(days=1),
                timedelta(hours=2),
                iceCapeMapping.GPS,
                None)

            logger.info("GPS job added")


    elif level == HIGH:
        logger.info("Installing  high jobs table")
        fake_base(gum, mode)
        if mode == REF:
            gum.add_job(
                MspJob.J_P_PCON,
                time(hour=11, minute=50),  # 1970-01-01 11:50:00
                timedelta(days=1),
                timedelta(hours=2),
                iceCapeMapping.WIFI_1 + iceCapeMapping.WIFI_2 + iceCapeMapping.WIFI_PA,
                None)
            logger.info("wifi job added")
    else:
        logger.error("not sure how I got here")
        raise GwLevelError("Unknown Level")
    #Power on 24/7 peripherals
    if level >= CRITICAL:
        logger.info("Powering on critical peripherals")
        if (mode == REF):
            gum.turn_on_sensor_output(iceCapeMapping.WEATHER)
            logger.info("Weather station turned on")
    if level >= LOW:
        logger.info("Powering on  low peripherals")
    if level >= OK:
        logger.info("Powering on ok peripherals")
    if level >= HIGH:
        logger.info("Powering on  high peripherals")
        if mode == BASE: # or mode == REF:
            logger.warn("GPS removed from Ref")
            gum.turn_on_sensor_output(iceCapeMapping.GPS)


def teardown(gum, level, mode):
    #generic tear down for all levels

    if level >= CRITICAL:
        logger.info("Powering off critical peripherals")
    if level >= LOW:
        logger.info("Powering off  low peripherals")
        if (mode == REF):
            gum.turn_off_sensor_output(iceCapeMapping.WEATHER)
        elif (mode == BASE):
            gum.turn_off_sensor_output(iceCapeMapping.GEOPHONE)
        elif mode is CAM:
            pass
        else:
            logger.error("NOT SURE HOW I GOT HERE")
            raise GwModeError()

    if level >= OK:
        logger.info("Powering off OK peripherals")

    if level >= HIGH:
        logger.info("Powering off  high peripherals")
        gum.turn_off_sensor_output(iceCapeMapping.GPS)
        if mode == REF:
            gum.turn_off_power_output(iceCapeMapping.WIFI_1 + iceCapeMapping.WIFI_2
                + iceCapeMapping.WIFI_PA)


def reset(gum):
    gum.output_off(65535)
