#!/usr/bin/env python

# Dumps the data from the weatherstation to a file in the data directory
# called <datetime>-weather

import weather
from variables import BASE_DIR, DATA_DIR 
from gwlogging import logger
import datetime
import sys

logger.info("Stopping weather recording")
ret = weather.stop()

if ret == 1 or ret < 0 or ret >= 126:
    logger.critical("Failed to stop recording")
    sys.exit(1)

logger.debug("Getting log file from console")
now = datetime.datetime.utcnow()
filename = BASE_DIR + DATA_DIR + "/%s-weather" % now.strftime("%Y-%m-%d-%H%M")
ret = weather.dump(filename)
if ret == 1 or ret < 0:
    logger.critical("Failed to dump weather data")
    sys.exit(2)
elif ret == 0:
    logger.debug("Weather data sucessfully dumped")
    logger.debug("Clearing weather console")
    ret = weather.clrlog()
    if ret == 1 or ret < 0:
        logger.critical("Failed to clear weather console")
        sys.exit(3)
    else:
        logger.debug("Weather console sucessfully cleared")

ret = weather.start()
if ret == 1 or ret < 0:
    logger.critical("Failed to restart weather console logging")
    sys.exit(4)
else:
    logger.debug("Weather console loggin restarted sucessfully")
    logger.info("Weather data sucessfully dumped")
