/**
 * Copyright (c) 2011, The University of Southampton and the individual contributors.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 *   * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 *
 *   *	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 *
 *   *	Neither the name of the University of Southampton nor the names of its
 * 	contributors may be used to endorse or promote products derived from this
 * 	software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef OpenIMAJGrabber_
#define OpenIMAJGrabber_

/* The classes below are exported */
#pragma GCC visibility push(default)

class Device {
    private:
        const char* name;
        const char* identifier;
    public:
        Device(const char* name, const char* identifier);
        ~Device();
        const char* getName();
        const char* getIdentifier();
};

class DeviceList {
private:
    int nDevices;
    Device** devices;
public:
    DeviceList(Device ** devices, int nDevices);
    ~DeviceList();
    int getNumDevices();
    Device * getDevice(int i);
};

class OpenIMAJGrabber
{
    public:
        OpenIMAJGrabber();
        ~OpenIMAJGrabber();

        DeviceList* getVideoDevices();

        unsigned char* getImage();
        void nextFrame();
        bool startSession(int width, int height, double rate );
        bool startSession(int width, int height, double rate, Device * device);
        void stopSession();

        int getWidth();
        int getHeight();
    private:
        void* data;
};

#pragma GCC visibility pop
#endif
